#! /usr/bin/env python                                                                                                                                                 
# ESA (C) 2000-2019
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#
#

# This script copies the pysas directory tree in
# $SAS_DIR/lib/python into the user's development
# environment, below the $SAS_DEV/lib/python.
# 
# Below $SAS_DIR/lib/python/pysas, all except
# the directories are relative soft links which
# point to .py modules included under the 
# respective src/<pkg> subdirectory, where <pkg>
# is any Python package.
#
# Below $SAS_DEV/lib/python/pysas, again, all
# except the dircetories will be absolute soft
# links.
#
# This script is run from the SAS Make.include
# under the target 'required_dirs'. Thus, besides
# the required directory tree to work with SAS
# which is created below $SAS_DEV, the subdirectory
# $SAS_DEV/lib/python/pysas, receives the whole
# contents of SAS Python, in terms of packages and 
# their modules.
#
# Why?
# pysas is the root Python package for SAS.
# To develop packages with SAS, the developer
# creates the directory $SAS_DEV, typically as
# $HOME/sasdev, which is placed ahead of $SAS_DIR.
# Such a directory arrangement allows to create in
# $SAS_DEV, a replica of all the standard
# directories found below $SAS_DIR, i.e. bin, lib,
# config, doc and include. As the developer builds
# new packages, the SAS Makefile creates below these
# directories, the package binaries (in $SAS_DEV/bin), 
# libraries ($SAS_DEV/lib), etc, all as soft links
# pointing to the real iles below the package
# directory tree. All these directories are placed 
# ahead similar directories below $SAS_DIR, 
# in the PATH, LD_LIBRARY_PATH, etc, env. variables.
# As a consequence, any new binaries, libraries, etc,
# produced by the developer during a local build of
# the package, are taken and used instead of those 
# already existing in the SAS build, below $SAS_DIR.
#
# For Python, the mechanism is similar. However, 
# Python packages are all sub-packages of pysas. 
# When the new package is built, its modules are 
# placed in a subdirectory of 
# $SAS_DEV/lib/python/pysas, named with the 
# package name. This way, it is possible to import 
# immediately the new package as a subpackge of pysas. 
# Given that $SAS_DEV/lib/python/pysas is placed 
# ahead of $SAS_DIR/lib/python/pysas, Python does
# not allow to fall back there to search for a given 
# package if not present in $SAS_DEV/lib/python/pysas.
# Thus, the only solution to allow the user test
# the whole SAS Python system is to replicate it below
# $SAS_DEV/lib/python.  Such replica is done 
# everytime the make is invoked with target 
# 'required_dirs' which also re-creates the rest of 
# the directory tree below $SAS_DEV.
# 
# Will use os.walk to run through the source dir
# tree.
#
# SAS_DIR and SAS_PATH must be defined, otherwise
# the procedure exits with an error message.
#
# SAS_DEV is the first element of SAS_PATH.
# The second is always SAS_DIR. However, we do not
# test here that both are the same.
#

# Standard library import
import os, sys, shutil


sasdir = os.environ.get('SAS_DIR')
if sasdir == None:
    print('Error: pyset.py: SAS_DIR is undefined')
    sys.exit()

saspath = os.environ.get('SAS_PATH')
if saspath == None:
    print('Error: pyset.py: SAS_PATH is undefined')
    sys.exit()

# sasdev corresponds to SAS_DEV, although this
# variable is not mandatory as are SAS_DIR and SAS_PATH      


sasdev = saspath.split(':')[0]

sasdirlibpysas = os.path.join(sasdir, 'lib/python/pysas')
sasdevlibpysas = os.path.join(sasdev, 'lib/python/pysas')

# Just to be able to return where we were
here = os.getcwd()

# Change to the SAS_DIR/lib/python/pysas source directory
os.chdir(sasdirlibpysas)


# os.walk is a generator object which can only be unfolded
# via a for loop.
#
# To scan the directory tree from a given point, we pass it
# as argument ('./'), and select a direction to scan with
# boolean 'topdown'. If True, it will go from top to down
# to find directories and files, ilike it does the ls 
# command, otherwise in the opposite direction.
#
# The result of the scan is a set of directories (cur_dir),
# where there could be subdirectories (sub_dirs) and files
# at the same level that these subdirectories.

for cur_dir, sub_dirs, files in os.walk('./', topdown=True):
    #print(f'\nProcessing {cur_dir}')
    if sub_dirs:
        #print('Subdirs: ', sub_dirs)
        for sd in sub_dirs:
            newdir = os.path.join(sasdevlibpysas, sd)
            #print(f'newdir = {newdir}')
            if os.path.exists(newdir):
                shutil.rmtree(newdir, ignore_errors=True)
            os.mkdir(newdir)
    #print('Files: ')
    for f in files:
        fullf = os.path.join(sasdirlibpysas, cur_dir.strip('./'), f)
        #print(f'\n{fullf}')
        linkf = os.readlink(fullf)
        absf = os.path.join(os.path.dirname(fullf), linkf)
        absf = os.path.abspath(absf)
        #print(absf)
        link = os.path.join(sasdevlibpysas, cur_dir.strip('./'), f)
        if os.path.exists(link):
            os.remove(link)
        #print(f'link = {link}')
        os.symlink(absf, link)

os.chdir(here)
